/*
 * grunt-plugin-build
 *
 *
 * Copyright (c) 2016 Jeroen de Lau
 * Licensed under the Proprietory license.
 */

'use strict';
var Zip = require('jszip');
var path = require('path');
var fs = require("fs");
var merge = require('merge');

module.exports = function (grunt) {

// Please see the Grunt documentation for more information regarding task
// creation: http://gruntjs.com/creating-tasks
    grunt.registerMultiTask(
        'plugin_build',
        'Copy and process source files to a build folder',
        function () {
            //Merge options
            var config = this.options({
                srcPath: 'src',
                buildPath: 'build',
                distPath: 'dist',
                manifestPath: '/manifest.json',
                replacements: {},
                omitUnused: false,
                publish: false,
                minify: false,
                mangle: true,
                includePaths: null,
                useFileCache: false
            });

            var publish = config.publish || grunt.option('publish');
            config.useFileCache = config.useFileCache || grunt.option('useFileCache');

            var callback = this.async();


            buildAlternative(false, "/source", function () {
                if (publish && config.minify)
                {
                    buildAlternative(true, "/min", callback);
                } else {
                    callback(true);
                }
            });

            function buildAlternative(min, path, cb)
            {
                var tmpConfig = merge(true, config);

                tmpConfig.minify = min;
                tmpConfig.buildPath = config.buildPath + path;
                grunt.log.oklns('Building extension: ' + tmpConfig.buildPath);
                var builder = new grunt.PluginBuilder.Builder(tmpConfig);
                builder.build(cb);
            }

        });


    grunt.registerMultiTask(
        'plugin_package',
        'Compress a package to distributable zip',
        function () {
            //search for packages
            var searchPath = 'build/';
            var distPath = 'dist';
            var destPath = '';

            //lookup manifests in the build folder
            var manifests = grunt.file.expand([searchPath + '**/manifest.json']);
            if (manifests.length === 0) {
                grunt.fail.fatal('No manifest files found in ' + searchPath);
                return;
            }

            grunt.log.oklns(manifests.length + ' manifests found in \'' + searchPath + '\'');

            //Loop through the manifests and create one zip for each
            if (!grunt.file.exists(distPath))
            {
                grunt.file.mkdir(distPath);
            }

            manifests.forEach(function (manifestPath)
            {
                var man = grunt.file.readJSON(manifestPath);
                var fileName = getName(manifestPath);
                var srcPath = manifestPath.replace('/manifest.json', '');

                destPath = distPath
                    + '/build-' + man.version
                    + '-' + grunt.config('pkg').buildnum;

                grunt.log.writelns('Zipping into: ' + destPath + '/' + fileName + ' ... ');

                //Create the zip
                var zip = new Zip();

                //get all files and add them to the zip
                grunt.file.recurse(
                    srcPath,
                    function (abspath, rootdir, subdir, filename)
                    {
                        var zipPath = subdir ? subdir + '/' : '';
                        zipPath += filename;

                        grunt.verbose.writeln('Adding file: ' + zipPath);

                        var content = fs.readFileSync(abspath);
                        zip.file(zipPath, content);
                    });
                var dest = path.resolve(destPath);
                fs.mkdir(dest);
                var output = zip.generate({base64: false, compression: 'DEFLATE'});
                fs.writeFileSync(dest + '/' + fileName, output, 'binary');


                //function to retrieve the namels
                function getName(manifestPath)
                {
                    var name =  man.short_name;
                    //get thel o
                    if(name.indexOf('__MSG')=== 0){
                        name = getLocalizedName(name);
                    }
                    var path = name + '-' + man.version + '';
                    path += '.' + manifestPath
                        .replace('/manifest.json', '.zip')
                        .replace(searchPath, '')
                        .replace(/\//g, '.');
                    return path;
                }

                function getLocalizedName(key)
                {
                    var key = key.match(/__MSG_(.*?)__/)[1];
                    var lang = man.default_locale;
                    var path = manifestPath.replace('manifest.json', "_locales/"+lang+"/messages.json")

                    var translations = grunt.file.readJSON(path);
                    return translations[key].message;
                }

            });

            //Copy the changlog
            grunt.log.write('Add changelog ...');
            if (grunt.file.exists('src/changelog.txt')) {
                grunt.file.copy("src/changelog.txt", destPath + '/changelog.txt');
                grunt.log.ok();
            } else {
                grunt.log.error("not found");
            }

        });

    grunt.registerTask(
        'plugin_increment_version',
        'Increments the manifest file by one',
        function () {

            //Merge options
            var config = this.options({
                srcPath: 'src',
                manifestPath: '/manifest.json'
            });

            var builder = new grunt.PluginBuilder.Builder(config);
            var man = builder.getManifest();
            var version = man.version.split('.');
            version[0] = version[0] || 0;
            version[1] = version[1] || 0;
            version[2] = version[2] || 0;
            version[2]++;
            man.version = version.join('.');

            grunt.file.write(
                builder.getSrcPath() + '/manifest.json',
                JSON.stringify(man, null, 2));

            grunt.log.oklns('Updated manifest version to > ' + man.version);

        });

    grunt.registerTask(
        'plugin_prep_dist',
        function () {
            grunt.option('publish', true);
        });

    grunt.registerTask(
        'plugin_fast_build',
        function () {
            grunt.option('useFileCache', true);
        });

    grunt.registerTask(
        'plugin_link_framework',
        function () {

            //Merge options
            var config = this.options({
                srcPath: 'src',
                frameworkPath: 'crazyroosters/'
            });

            var builder = new grunt.PluginBuilder.Builder(config);
            var man = builder.getManifest();

            grunt.file.write(
                builder.getSrcPath() + '/manifest.json',
                JSON.stringify(man, null, 2));

            grunt.log.oklns('Updated manifest version to > ' + man.version);

        });
};
